@@ -39,12 +39,40 @@
 
 
 /**
- * Implementation of the Connection/Delegate interface for postgresql. 
- * 
+ * Implementation of the Connection/Delegate interface for PostgreSQL.
+ *
+ * Note: Due to design limitations in both libpq and PostgreSQL's
+ * architecture, this implementation does not support the superior
+ * memory-bounded streaming behavior we use with MySQL and Oracle.
+ *
+ * The core issue is that libpq's PQexec() buffers entire result sets in
+ * client memory before returning, even when the PostgreSQL server is
+ * streaming tuples over the socket. This is a design choice, not a
+ * fundamental protocol limitation - libpq simply waits to receive all
+ * rows before giving control back to the application. This means:
+ * - Result sets are fully buffered on the client side
+ * - For queries requiring server-side materialization (ORDER BY, GROUP BY,
+ *   etc.), the result set is allocated TWICE: once on the server and once
+ *   in libpq, potentially doubling memory consumption
+ * - Large queries cannot be aborted mid-execution to free server resources
+ *
+ * While PostgreSQL does support server-side cursors (DECLARE/FETCH), they
+ * are poorly suited for a general-purpose connection library: they require
+ * an explicit transaction context, each fetch is a separate query round-trip,
+ * and they lack the transparent, efficient cursor support that MySQL and
+ * Oracle provide at the protocol level.
+ *
+ * libpq's PQsetSingleRowMode() only addresses client-side buffering and
+ * does nothing for server-side memory usage or query abort capability.
+ *
+ * On the positive side, this simpler execution model means libzdb works
+ * reliably with PostgreSQL connection proxies (pgBouncer, PgPool-II, etc.)
+ * when using simple queries. Note that prepared statements may still
+ * experience compatibility issues with certain proxy configurations.
+ *
  * @file
  */
 
-
 /* ----------------------------------------------------------- Definitions */
 
 
@@ -101,7 +129,7 @@ static bool _doConnect(T C, char **error) {
         else
                 ERROR("no database specified in URL");
         /* SSL Options */
-        StringBuffer_append(C->sb, "sslmode='%s' ", IS(URL_getParameter(url, "use-ssl"), "true") ? "require" : "disable");
+        StringBuffer_append(C->sb, "sslmode='%s' ", Str_parseBool(URL_getParameter(url, "use-ssl")) ? "require" : "disable");
         if (URL_getParameter(url, "ssl-ca")) {
                 StringBuffer_append(C->sb, "sslrootcert='%s' ", URL_getParameter(url, "ssl-ca"));
         }
@@ -113,11 +141,7 @@ static bool _doConnect(T C, char **error) {
         }
         /* Other Options */
         if (URL_getParameter(url, "connect-timeout")) {
-                TRY
-                        StringBuffer_append(C->sb, "connect_timeout=%d ", Str_parseInt(URL_getParameter(url, "connect-timeout")));
-                ELSE
-                        ERROR("invalid connect timeout value");
-                END_TRY;
+                StringBuffer_append(C->sb, "connect_timeout=%d ", Str_parseInt(URL_getParameter(url, "connect-timeout")));
         } else
                 StringBuffer_append(C->sb, "connect_timeout=%lld ", SQL_DEFAULT_TIMEOUT/MSEC_PER_SEC);
         if (URL_getParameter(url, "application-name"))
@@ -161,7 +185,9 @@ static T _new(Connection_T delegator, char **error) {
 
 static bool _ping(T C) {
         assert(C);
-        return (PQstatus(C->db) == CONNECTION_OK);
+        PQclear(C->res);
+        C->res = PQexec(C->db, "");
+        return (PQresultStatus(C->res) == PGRES_EMPTY_QUERY);
 }
 
 
@@ -188,33 +214,34 @@ static bool _beginTransactionType(T C, TRANSACTION_TYPE type) {
                 default:
                         sql = "BEGIN TRANSACTION;";
         }
-        PGresult *res = PQexec(C->db, sql);
-        C->lastError = PQresultStatus(res);
-        PQclear(res);
+        PQclear(C->res);
+        C->res = PQexec(C->db, sql);
+        C->lastError = PQresultStatus(C->res);
         return (C->lastError == PGRES_COMMAND_OK);
 }
 
 
 static bool _commit(T C) {
 	assert(C);
-        PGresult *res = PQexec(C->db, "COMMIT TRANSACTION;");
-        C->lastError = PQresultStatus(res);
-        PQclear(res);
+        PQclear(C->res);
+        C->res = PQexec(C->db, "COMMIT TRANSACTION;");
+        C->lastError = PQresultStatus(C->res);
         return (C->lastError == PGRES_COMMAND_OK);
 }
 
 
 static bool _rollback(T C) {
 	assert(C);
-        PGresult *res = PQexec(C->db, "ROLLBACK TRANSACTION;");
-        C->lastError = PQresultStatus(res);
-        PQclear(res);
+        PQclear(C->res);
+        C->res = PQexec(C->db, "ROLLBACK TRANSACTION;");
+        C->lastError = PQresultStatus(C->res);
         return (C->lastError == PGRES_COMMAND_OK);
 }
 
 
 static long long _lastRowId(T C) {
         assert(C);
+        // NA: See Connection_lastRowId documentation
         return (long long)PQoidValue(C->res);
 }
 
@@ -276,10 +303,17 @@ static PreparedStatement_T _prepareStatement(T C, const char *sql, va_list ap) {
 
 static const char *_getLastError(T C) {
 	assert(C);
-        return C->res ? PQresultErrorMessage(C->res) : "unknown error";
+        return _getSQLErrorMessage(C->res);
 }
 
 
+static int _getLastErrorCode(T C) {
+        assert(C);
+        // Return SQLSTATE if available, otherwise 0
+        // We intentionally do NOT mix in PQresultStatus codes
+        return _getSQLStateErrorCode(C->res);
+}
+
 /* ------------------------------------------------------------------------- */
 
 
@@ -297,6 +331,7 @@ const struct Cop_T postgresqlcops = {
         .execute                = _execute,
         .executeQuery           = _executeQuery,
         .prepareStatement       = _prepareStatement,
-        .getLastError           = _getLastError
+        .getLastError           = _getLastError,
+        .getLastErrorCode       = _getLastErrorCode
 };
 
