@@ -83,8 +83,9 @@ static void _setString(T P, int parameterIndex, const char *x, int size) {
         assert(P);
         sqlite3_reset(P->stmt);
         P->lastError = sqlite3_bind_text(P->stmt, parameterIndex, x, size, SQLITE_STATIC);
-        if (P->lastError == SQLITE_RANGE)
+        if (P->lastError == SQLITE_RANGE) {
                 THROW(SQLException, "Parameter index is out of range");
+        }
 }
 
 
@@ -92,8 +93,9 @@ static void _setInt(T P, int parameterIndex, int x) {
         assert(P);
         sqlite3_reset(P->stmt);
         P->lastError = sqlite3_bind_int(P->stmt, parameterIndex, x);
-        if (P->lastError == SQLITE_RANGE)
+        if (P->lastError == SQLITE_RANGE) {
                 THROW(SQLException, "Parameter index is out of range");
+        }
 }
 
 
@@ -101,8 +103,9 @@ static void _setLLong(T P, int parameterIndex, long long x) {
         assert(P);
         sqlite3_reset(P->stmt);
         P->lastError = sqlite3_bind_int64(P->stmt, parameterIndex, x);
-        if (P->lastError == SQLITE_RANGE)
+        if (P->lastError == SQLITE_RANGE) {
                 THROW(SQLException, "Parameter index is out of range");
+        }
 }
 
 
@@ -110,8 +113,9 @@ static void _setDouble(T P, int parameterIndex, double x) {
         assert(P);
         sqlite3_reset(P->stmt);
         P->lastError = sqlite3_bind_double(P->stmt, parameterIndex, x);
-        if (P->lastError == SQLITE_RANGE)
+        if (P->lastError == SQLITE_RANGE) {
                 THROW(SQLException, "Parameter index is out of range");
+        }
 }
 
 
@@ -119,8 +123,9 @@ static void _setTimestamp(T P, int parameterIndex, time_t x) {
         assert(P);
         sqlite3_reset(P->stmt);
         P->lastError = sqlite3_bind_int64(P->stmt, parameterIndex, x);
-        if (P->lastError == SQLITE_RANGE)
+        if (P->lastError == SQLITE_RANGE) {
                 THROW(SQLException, "Parameter index is out of range");
+        }
 }
 
 
@@ -128,8 +133,9 @@ static void _setBlob(T P, int parameterIndex, const void *x, int size) {
         assert(P);
         sqlite3_reset(P->stmt);
         P->lastError = sqlite3_bind_blob(P->stmt, parameterIndex, x, size, SQLITE_STATIC);
-        if (P->lastError == SQLITE_RANGE)
+        if (P->lastError == SQLITE_RANGE) {
                 THROW(SQLException, "Parameter index is out of range");
+        }
 }
 
 
@@ -138,15 +144,15 @@ static void _execute(T P) {
         P->lastError = zdb_sqlite3_step(P->stmt);
         switch (P->lastError) {
                 case SQLITE_DONE:
-                        P->lastError = sqlite3_reset(P->stmt);
+                        sqlite3_reset(P->stmt);
                         break;
                 case SQLITE_ROW:
-                        P->lastError = sqlite3_reset(P->stmt);
+                        sqlite3_reset(P->stmt);
                         THROW(SQLException, "Select statement not allowed in PreparedStatement_execute()");
                         break;
                 default:
-                        P->lastError = sqlite3_reset(P->stmt);
-                        THROW(SQLException, "Connection [%p] %s", P->delegator, sqlite3_errmsg(P->db));
+                        sqlite3_reset(P->stmt);
+                        THROW_SQL(sqlite3_extended_errcode(P->db), "Connection [%p] %s", P->delegator, sqlite3_errmsg(P->db));
                         break;
         }
 }
@@ -156,7 +162,7 @@ static ResultSet_T _executeQuery(T P) {
         assert(P);
         if (P->lastError == SQLITE_OK)
                 return ResultSet_new(SQLiteResultSet_new(P->delegator, P->stmt, true), (Rop_T)&sqlite3rops);
-        THROW(SQLException, "Connection [%p] %s", P->delegator, sqlite3_errmsg(P->db));
+        THROW_SQL(sqlite3_extended_errcode(P->db), "Connection [%p] %s", P->delegator, sqlite3_errmsg(P->db));
         return NULL;
 }
 
