@@ -51,7 +51,7 @@ void *Mem_alloc(long size, const char *func, const char *file, int line){
 	assert(size > 0);
 	void *p = malloc(size);
 	if (! p)
-		Exception_vthrow(&MemoryException, func, file, line, "%s", System_getLastError());
+		Exception_vthrow(&MemoryException, 0, func, file, line, "%s", System_getLastError());
 	return p;
 }
 
@@ -61,7 +61,7 @@ void *Mem_calloc(long count, long size, const char *func, const char *file, int
 	assert(size > 0);
 	void *p = calloc(count, size);
 	if (! p)
-		Exception_vthrow(&MemoryException, func, file, line, "%s", System_getLastError());
+		Exception_vthrow(&MemoryException, 0, func, file, line, "%s", System_getLastError());
 	return p;
 }
 
@@ -77,7 +77,7 @@ void *Mem_resize(void *p, long size, const char *func, const char *file, int lin
 	assert(size > 0);
 	p = realloc(p, size);
 	if (! p)
-		Exception_vthrow(&MemoryException, func, file, line, "%s", System_getLastError());
+		Exception_vthrow(&MemoryException, 0, func, file, line, "%s", System_getLastError());
 	return p;
 }
 
