@@ -3,12 +3,12 @@
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
@@ -44,14 +44,14 @@
 
 
 static inline bool _is_equals_ci(const char *s, const char *literal) {
-    for (int i = 0; ; i++) {
-        if (literal[i] == 0) {
-            return s[i] == 0 || isspace((unsigned char)s[i]);
+        for (int i = 0; ; i++) {
+                if (literal[i] == 0) {
+                        return s[i] == 0 || isspace((uchar_t)s[i]);
+                }
+                if (tolower((uchar_t)s[i]) != literal[i]) { // literal already lowercase
+                        return false;
+                }
         }
-        if (tolower((unsigned char)s[i]) != tolower((unsigned char)literal[i])) {
-            return false;
-        }
-    }
 }
 
 
@@ -63,36 +63,31 @@ static inline bool _is_equals_ci(const char *s, const char *literal) {
 #endif
 
 bool Str_isEqual(const char *a, const char *b) {
-	if (a && b) { 
-                while (*a && *b)
-                        if (toupper(*a++) != toupper(*b++)) return false;
-                return (*a == *b);
-        }
+        if (a && b)
+                return (strcasecmp(a, b) == 0);
         return false;
 }
 
 
 bool Str_isByteEqual(const char *a, const char *b) {
-	if (a && b) {
-                while (*a && *b)
-                        if (*a++ != *b++) return false;
-                return (*a == *b);
-        }
+        if (a && b)
+                return (__builtin_strcmp(a, b) == 0);
         return false;
 }
 
 
 bool Str_startsWith(const char *a, const char *b) {
-        if (a && b) {
-                do {
-                        if (*a != *b)
-                                return false;
-                        if (*a++ == 0 || *b++ == 0)
-                                break;
-                } while (*b);
-                return true;
-        }
-        return false;
+        if (!STR_DEF(a) || !STR_DEF(b))
+                return false;
+        size_t b_len = strlen(b);
+        return strncasecmp(a, b, b_len) == 0;
+}
+
+
+char *Str_sub(const char *a, const char *b) {
+        if (!a || !STR_DEF(b))
+                return NULL;
+        return strcasestr(a, b);
 }
 
 
@@ -108,19 +103,19 @@ bool Str_member(const char *s, const char **set) {
 
 
 char *Str_copy(char *dest, const char *src, int n) {
-	if (src && dest && (n > 0)) { 
-        	char *t = dest;
-	        while (*src && n--)
-        		*t++ = *src++;
-        	*t = 0;
-	} else if (dest)
-	        *dest = 0;
+        if (src && dest && (n > 0)) {
+                char *t = dest;
+                while (*src && n--)
+                        *t++ = *src++;
+                *t = 0;
+        } else if (dest)
+                *dest = 0;
         return dest;
 }
 
 
 // We do not use strdup so we can throw MemoryException on OOM
-char *Str_dup(const char *s) { 
+char *Str_dup(const char *s) {
         char *t = NULL;
         if (s) {
                 size_t n = strlen(s) + 1;
@@ -135,7 +130,7 @@ char *Str_ndup(const char *s, int n) {
         char *t = NULL;
         assert(n >= 0);
         if (s) {
-                int l = (int)strlen(s); 
+                int l = (int)strlen(s);
                 n = l < n ? l : n; // Use the actual length of s if shorter than n
                 t = ALLOC(n + 1);
                 memcpy(t, s, n);
@@ -146,14 +141,14 @@ char *Str_ndup(const char *s, int n) {
 
 
 char *Str_cat(const char *s, ...) {
-	char *t = 0;
-	if (s) {
+        char *t = 0;
+        if (s) {
                 va_list ap;
                 va_start(ap, s);
                 t = Str_vcat(s, ap);
                 va_end(ap);
         }
-	return t;
+        return t;
 }
 
 
@@ -174,47 +169,47 @@ char *Str_vcat(const char *s, va_list ap) {
 
 
 int Str_parseInt(const char *s) {
-	if (STR_UNDEF(s))
-		THROW(SQLException, "NumberFormatException: For input string null");
+        if (STR_UNDEF(s))
+                THROW(SQLException, "NumberFormatException: For input string null");
         errno = 0;
         char *e;
-	int i = (int)strtol(s, &e, 10);
-	if (errno || (e == s))
-		THROW(SQLException, "NumberFormatException: For input string %s -- %s", s, System_getLastError());
-	return i;
+        int i = (int)strtol(s, &e, 10);
+        if (errno || (e == s))
+                THROW(SQLException, "NumberFormatException: For input string %s -- %s", s, System_getLastError());
+        return i;
 }
 
 
 long long Str_parseLLong(const char *s) {
-	if (STR_UNDEF(s))
-		THROW(SQLException, "NumberFormatException: For input string null");
+        if (STR_UNDEF(s))
+                THROW(SQLException, "NumberFormatException: For input string null");
         errno = 0;
         char *e;
-	long long ll = strtoll(s, &e, 10);
-	if (errno || (e == s))
-		THROW(SQLException, "NumberFormatException: For input string %s -- %s", s, System_getLastError());
-	return ll;
+        long long ll = strtoll(s, &e, 10);
+        if (errno || (e == s))
+                THROW(SQLException, "NumberFormatException: For input string %s -- %s", s, System_getLastError());
+        return ll;
 }
 
 
 double Str_parseDouble(const char *s) {
-	if (STR_UNDEF(s))
-		THROW(SQLException, "NumberFormatException: For input string null");
+        if (STR_UNDEF(s))
+                THROW(SQLException, "NumberFormatException: For input string null");
         errno = 0;
         char *e;
-	double d = strtod(s, &e);
-	if (errno || (e == s))
-		THROW(SQLException, "NumberFormatException: For input string %s -- %s", s, System_getLastError());
-	return d;
+        double d = strtod(s, &e);
+        if (errno || (e == s))
+                THROW(SQLException, "NumberFormatException: For input string %s -- %s", s, System_getLastError());
+        return d;
 }
 
 
 bool Str_parseBool(const char *s) {
         if (STR_DEF(s)) {
-                while (isspace((unsigned char)*s)) s++;
-                switch (tolower((unsigned char)*s)) {
+                while (isspace((uchar_t)*s)) s++;
+                switch (tolower((uchar_t)*s)) {
                         case '1':
-                                return s[1] == '\0' || isspace(s[1]);
+                                return s[1] == '\0' || isspace((uchar_t)s[1]);
                         case 'y':
                                 return _is_equals_ci(s, "yes");
                         case 't':
@@ -231,7 +226,6 @@ bool Str_parseBool(const char *s) {
 }
 
 
-
 #ifdef PACKAGE_PROTECTED
 #pragma GCC visibility pop
 #endif
