@@ -186,7 +186,7 @@
  *     // Connection is automatically returned to pool when it goes out of scope
  *     // If an exception occurred before commit, it will automatically rollback
  * } catch (const sql_exception& e) {
- *     std::cerr << "Transfer failed: " << e.what() << std::endl;
+ *     std::cerr << "Transfer failed (" << e.error_code << "): << e.what() << std::endl;
  * }
  * ```
  * Key points about exception handling in this library:
@@ -219,7 +219,7 @@ namespace zdb {
     
     namespace { // private
         // @cond hide
-        #define except_wrapper(f) TRY { f; } ELSE { throw sql_exception(Exception_frame.message); } END_TRY
+        #define except_wrapper(f) TRY { f; } ELSE { throw sql_exception(Exception_frame.message, Exception_frame.errorCode); } END_TRY
         
         constexpr std::optional<std::string_view> _to_optional(const char* str) noexcept {
             return str ? std::optional<std::string_view>{str} : std::nullopt;
@@ -255,31 +255,60 @@ namespace zdb {
         // @endcond
     } // anonymous namespace
     
-    
     /**
      * @brief Exception class for SQL related errors.
      *
-     * Thrown for SQL errors. Inherits from `std::runtime_error`.
+     * Thrown for API and SQL errors. Inherits from `std::runtime_error`.
      *
-     * Example:
+     * The `error_code` member provides the native error code from the underlying
+     * database, enabling programmatic error handling such as retry logic for
+     * deadlocks or special handling for constraint violations. Error codes are
+     * database-specific:
+     *
+     * - **PostgreSQL**: SQLSTATE codes encoded as integers (use constants from SQLState.h)
+     * - **MySQL/MariaDB**: Native error codes from `mysql_errno()` (include `<mysqld_error.h>`)
+     * - **SQLite**: Extended error codes (include `<sqlite3.h>`)
+     * - **Oracle**: ORA- error numbers (numeric portion only)
+     *
+     * Example (PostgreSQL):
      * @code
      * try {
-     *     con.executeQuery("invalid query");
+     *     con.execute(sql);
      * } catch (const zdb::sql_exception& e) {
-     *     std::cout << "SQL error: " << e.what() << std::endl;
+     *     if (e.error_code == SQLSTATE_unique_violation) {
+     *         // Handle duplicate key
+     *     } else if (e.error_code == SQLSTATE_foreign_key_violation) {
+     *         // Handle FK constraint failure
+     *     } else if (e.error_code == SQLSTATE_deadlock_detected) {
+     *         // Retry transaction
+     *     } else {
+     *         std::cerr << "Error (" << e.error_code << "): " << e.what() << std::endl;
+     *     }
      * }
      * @endcode
+     *
+     * For MySQL, SQLite, and Oracle, include the respective database header
+     * with error codes or use numeric codes directly.
      */
     class sql_exception : public std::runtime_error {
     public:
         /**
-         * @brief Constructs a new sql_exception with an optional error message.
-         * @param msg A C-string representing the error message. Defaults to "SQLException".
-         */
-        explicit sql_exception(const char* msg = "SQLException") : std::runtime_error(msg) {}
+        * @brief Native error code from the underlying database.
+        *
+        * The value is `0` if no specific error code was provided. Consult your
+        * database's documentation for code meanings.
+        */
+        const int error_code;
+        
+        /**
+        * @brief Constructs a new sql_exception.
+        * @param msg Error message (defaults to "SQLException")
+        * @param code Database-specific error code (defaults to 0)
+        */
+        explicit sql_exception(const char* msg = "SQLException", int code = 0)
+        : std::runtime_error(msg), error_code(code) {}
     };
-    
-    
+
     /**
      * @class URL
      * @brief Represents an immutable Uniform Resource Locator.
@@ -692,6 +721,20 @@ namespace zdb {
             except_wrapper(RETURN ResultSet_isnull(t_, columnIndex));
         }
         
+        /**
+         * @brief Checks if the designated column's value is SQL NULL.
+         *
+         * A ResultSet returns an optional for reference types and 0 for value types.
+         * Use this method if you need to differentiate between SQL NULL and std::nullopt/0.
+         *
+         * @param columnName The SQL name of the column. case-sensitive.
+         * @return true if column value is SQL NULL, false otherwise.
+         * @throws sql_exception If a database access error occurs or columnName does not exist.
+         */
+        [[nodiscard]] bool isNull(const std::string& columnName) {
+            except_wrapper(RETURN ResultSet_isnullByName(t_, columnName.c_str()));
+        }
+        
         /**
          * @brief Gets the designated column's value as a string.
          *
@@ -1034,7 +1077,7 @@ namespace zdb {
      *     con.commit();
      * } catch (const sql_exception& e) {
      *     con.rollback();
-     *     std::cerr << "Database error: " << e.what() << std::endl;
+     *     std::cerr << "Database error (" << e.error_code << "): << e.what() << std::endl;
      * }
      * ```
      *
@@ -1300,7 +1343,7 @@ namespace zdb {
      *     std::cout << "Transfer successful" << std::endl;
      * } catch (const sql_exception& e) {
      *     // See note below why we don't have to explicit call rollback here
-     *     std::cerr << "Transfer failed: " << e.what() << std::endl;
+     *     std::cerr << "Transfer failed (" << e.error_code << "): << e.what() << std::endl;
      * }
      * @endcode
      *
@@ -1350,8 +1393,7 @@ namespace zdb {
          * @brief Sets the query timeout for this Connection.
          *
          * If the limit is exceeded, the statement will return immediately with an error.
-         * The timeout is set per connection/session. Not all database systems
-         * support query timeout. The default is no query timeout.
+         * The timeout is set per connection/session. The default is no query timeout.
          *
          * @param ms Timeout in milliseconds.
          */
@@ -1751,13 +1793,9 @@ namespace zdb {
      *
      * The pool is designed to dynamically manage the number of active connections
      * based on usage patterns. A `reaper` thread is automatically started when the
-     * pool is initialized, performing two functions:
-     *
-     * 1. Sweep through the pool at regular intervals (default every 60 seconds)
-     *    to close connections that have been inactive for a specified time (default
-     *    90 seconds).
-     * 2. Perform periodic validation (ping test) on idle connections to ensure
-     *    they remain valid and responsive.
+     * pool is initialized to sweep through the pool at regular intervals (default
+     * every 60 seconds) and close connections that have been inactive for a specified
+     * time (default 90 seconds).
      *
      * ## Realtime inspection:
      *
@@ -1931,14 +1969,14 @@ namespace zdb {
          *
          * The reaper thread closes inactive Connections in the pool, down to the
          * initial connection count. An inactive Connection is closed if its
-         * `connectionTimeout` has expired or if it fails a ping test. Active
-         * Connections (those in current use) are never closed by this thread.
+         * `connectionTimeout` has expired. Active Connections (those in current use)
+         * are never closed by this thread.
          *
          * This method can be called before or after ConnectionPool::start(). If
          * called after start, the changes will take effect on the next sweep cycle.
          *
          * @param sweepInterval Number of seconds between sweeps of the reaper thread.
-         *        Set to 0 or a negative value to disable the reaper thread, _before_
+         *        Set to 0 or a negative value to disable the reaper thread, **before**
          *        calling ConnectionPool::start().
          */
         void setReaper(int sweepInterval) noexcept {
