@@ -90,7 +90,7 @@ static void testStr(void) {
                 assert(! Str_startsWith(NULL, "mysql"));
                 assert(! Str_startsWith("", NULL));
                 assert(! Str_startsWith(NULL, NULL));
-                assert(Str_startsWith("", ""));
+                assert(!Str_startsWith("", ""));
         }
         printf("=> Test5: OK\n\n");
         
@@ -149,6 +149,16 @@ static void testStr(void) {
         }
         printf("=> Test7: OK\n\n");
 
+        printf("=> Test8: sub-string\n");
+        {
+                assert(Str_isEqual(Str_sub("Haystack with Needle", "needle"), "Needle"));
+                assert(!Str_sub("Haystack with Needle", "hay stack"));
+                assert(Str_sub("MySQL 8.0.35-router", "router") != NULL);
+                assert(Str_sub("5.7.42-MaxScale", "maxscale") != NULL);
+                assert(Str_sub("8.0.33", "router") == NULL);
+        }
+        printf("=> Test8: OK\n\n");
+
         
         printf("============> Str Tests: OK\n\n");
 }
@@ -931,16 +941,12 @@ static void testStringBuffer(void) {
                 assert(Str_isEqual(StringBuffer_toString(sb), "select a from b"));
                 StringBuffer_free(&sb);
                 assert(sb == NULL);
-                // Remove last semicolon
-                sb = StringBuffer_new("select * from host;");
+                // Keep last semicolon
+                sb = StringBuffer_new("select * from host; ");
                 StringBuffer_trim(sb);
-                assert(Str_isEqual(StringBuffer_toString(sb), "select * from host"));
+                assert(Str_isEqual(StringBuffer_toString(sb), "select * from host;"));
                 StringBuffer_free(&sb);
-                sb = StringBuffer_new(";");
-                StringBuffer_trim(sb);
-                assert(Str_isEqual(StringBuffer_toString(sb), ""));
-                StringBuffer_free(&sb);
-                // Test don't remove last semicolon if part of 'end;'
+                // Keep last semicolon
                 sb = StringBuffer_new("DECLARE blabla END; \n");
                 StringBuffer_trim(sb);
                 assert(Str_isEqual(StringBuffer_toString(sb), "DECLARE blabla END;"));
@@ -951,7 +957,7 @@ static void testStringBuffer(void) {
                 StringBuffer_free(&sb);
                 sb = StringBuffer_new("select * from x order by id; \t\n");
                 StringBuffer_trim(sb);
-                assert(Str_isEqual(StringBuffer_toString(sb), "select * from x order by id"));
+                assert(Str_isEqual(StringBuffer_toString(sb), "select * from x order by id;"));
                 StringBuffer_free(&sb);
                 sb = StringBuffer_new("end;");
                 StringBuffer_trim(sb);
@@ -959,17 +965,17 @@ static void testStringBuffer(void) {
                 StringBuffer_free(&sb);
                 sb = StringBuffer_new("and; \t\n");
                 StringBuffer_trim(sb);
-                assert(Str_isEqual(StringBuffer_toString(sb), "and"));
+                assert(Str_isEqual(StringBuffer_toString(sb), "and;"));
                 StringBuffer_free(&sb);
                 sb = StringBuffer_new("nd;");
                 StringBuffer_trim(sb);
-                assert(Str_isEqual(StringBuffer_toString(sb), "nd"));
+                assert(Str_isEqual(StringBuffer_toString(sb), "nd;"));
                 StringBuffer_free(&sb);
                 assert(sb == NULL);
                 // Remove white space
                 sb = StringBuffer_new("\t select a from b; \r\n");
                 StringBuffer_trim(sb);
-                assert(Str_isEqual(StringBuffer_toString(sb), "select a from b"));
+                assert(Str_isEqual(StringBuffer_toString(sb), "select a from b;"));
                 StringBuffer_free(&sb);
                 assert(sb == NULL);
         }
